@@ -4,7 +4,7 @@ shinyUI(pageWithSidebar(
   headerPanel("Uploading Files"),
   sidebarPanel(
     fileInput('file1', 'Choose CSV File',
-              accept=c('text/csv', 'text/comma-separated-values,text/plain')),
+              accept=c('text/csv', 'text/comma-separated-values,text/plain', '.csv')),
     tags$hr(),
     checkboxInput('header', 'Header', TRUE),
     radioButtons('sep', 'Separator',
